package com.ruoyi.sim.service.impl;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.sim.config.SimConfig;
import com.ruoyi.sim.domain.Seat;
import com.ruoyi.sim.domain.vo.SimSocketVo;
import com.ruoyi.sim.domain.vo.SocketWrapValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.Socket;
import java.util.HashMap;
import java.util.List;

import static com.ruoyi.sim.constant.CommConst.SOCKET_TIME_OUT;

/**
 * 1.所有Socket常开。
 * todo: connectedTimeMillis 时间。
 */
@Service
public class SocketService {

    private static final Logger l = LoggerFactory.getLogger(SocketService.class);

    private static final int INIT_SIZE = 32;
    /**
     * 6 hours
     * 1000L * 60 * 60 * 6
     * 1000L * 60 * 5
     */
    private static final long TIMEOUT_LIMIT = 1000L * 60 * 60 * 6;

    /**
     * key: ip:port
     * value:
     */
    private static HashMap<String, SocketWrapValue> cachedMap = new HashMap<>(INIT_SIZE);

    @Autowired
    private SimConfig config;
    @Autowired
    private FailedCountService failedCountService;
    @Autowired
    private SeatService seatService;

    /**
     * @param ssv
     * @return true:socket ok!
     */
    public boolean isOk(final SimSocketVo ssv) {
        final String key = ssv.toKey();
        if (cachedMap.containsKey(key) && cachedMap.get(key) != null) {
            Socket s = cachedMap.get(key).getSocket();
            if (s != null) {
                return (s.isConnected() && s.isBound() && !s.isClosed());
            }
        }
        return false;
    }

    public boolean isNotOk(final SimSocketVo ssv) {
        return !isOk(ssv);
    }

    /**
     * 暂时不考虑超时周期。
     * todo:
     *
     * @param ssv
     * @return
     */
    public boolean isTimeout(SimSocketVo ssv) {
        if (cachedMap.containsKey(ssv.toKey())) {
            Long cached = cachedMap.get(ssv.toKey()).getOkTimeMillis();
            if (cached == null || cached == 0L) {
                return true;
            }
            return System.currentTimeMillis() - cached <= TIMEOUT_LIMIT;
        }
        return true;
    }

    /**
     * @param ssv
     * @return
     */
    public AjaxResult openOne(final SimSocketVo ssv) {
        // check.
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        //
        try {
            if (isNotOk(ssv)) {
                final String key = ssv.toKey();
                l.info("openSocket cachedSocket is not ok!try new socket ip = {}:{}!", ssv.getIp(), ssv.getPort());
                closeOne(ssv);
                Socket s = new Socket(ssv.getIp(), ssv.getPort());
                s.setSoTimeout(SOCKET_TIME_OUT);
                SocketWrapValue value = new SocketWrapValue(ssv.getIp(), ssv.getPort(), s, System.currentTimeMillis());
                cachedMap.put(key, value);
                // failed count reset.
                failedCountService.reset0(ssv);
            } else {
                l.info("openSocket cachedSocket cache ok!cached socket ip = {}:{}!", ssv.getIp(), ssv.getPort());
            }
            Seat seat = seatService.uniqueByRs485IpAndPort(ssv.getIp(), ssv.getPort());
            seat.setSeatRs485SocketState(Seat.SocketState.ONLINE);
            seatService.updateSeat(seat);
            return AjaxResult.success("Socket[" + ssv.getIp() + ":" + ssv.getPort() + "],创建成功！");
        } catch (IOException e) {
            l.error(ssv.toString());
            return AjaxResult.error("Socket[" + ssv.getIp() + ":" + ssv.getPort() + "],创建失败！");
        }
    }

    /**
     * todo:部分返回Aj结果。
     *
     * @return
     */
    public AjaxResult tryOpenAll() {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        List<Seat> allSeat = seatService.listAllEnable();
        for (Seat s : allSeat) {
            openOne(s.toSimSocketVo());
        }
        return AjaxResult.success("所有Socket，创建成功！");
    }

    public AjaxResult closeOne(final SimSocketVo ssv) {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        String msgOk = "关闭Socket成功！";
        final String key = ssv.toKey();
        try {
            if (cachedMap.containsKey(key)) {
                Socket s = cachedMap.get(key).getSocket();
                s.getInputStream().close();
                s.getOutputStream().close();
                s.close();
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            cachedMap.remove(key);
            // failed count reset.
            failedCountService.reset0(ssv);
            return AjaxResult.success(msgOk);
        }
    }

    public AjaxResult closeAll() {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == {} [模拟器通信被禁用！]", config.isCommGlobal());
            return AjaxResult.error("模拟器通信被禁用！");
        }
        final String msgOk = "关闭所有Socket成功！";
        List<Seat> allSeat = seatService.listAllEnable();
        for (Seat s : allSeat) {
            closeOne(new SimSocketVo(s.getSeatRs485Ip(), s.getSeatRs485Port()));
        }
        failedCountService.resetAll();
        return AjaxResult.success(msgOk);
    }

    public Socket get(final Seat seat) {
        if (seat == null) {
            throw new IllegalArgumentException("seat is null");
        }
        return get(new SimSocketVo(seat.getSeatRs485Ip(), seat.getSeatRs485Port()));
    }

    /**
     * @param ssv
     * @return
     */
    public Socket get(final SimSocketVo ssv) {
        if (isNotOk(ssv)) {
            openOne(ssv);
        }
        return cachedMap.get(ssv.toKey()).getSocket();
    }

    /**
     * 初始化。
     */
    public void resetAll() {

    }
}
